*+
*+
*+    Source Module => FTPDIR.PRG
*+
*+    Public Domain from Phil Barnett.
*+
*+    Functions: Procedure ftpdir()
*+               Procedure readdir()
*+               Function convert()
*+               Function token()
*+               Function numtoken()
*+               Function tokenize()
*+               Function stod()
*+               Function gdateformt()
*+               Procedure sdateformt()
*+
*+       Tables: use ( dbfname ) exclusive new alias thetable
*+
*+    Reformatted by Click! 1.13 on Jan-14-1998 at 10:37 pm
*+
*+

#include "COMMON.CH"
#include "SET.CH"

*+
*+
*+    Procedure ftpdir()
*+
*+
*+
procedure ftpdir( ip, login, password, path, dbfname )

local cInstruct := '$inst.ftp'
local cDirectry := '$dir.ftp'
local CRLF      := chr( 13 ) + chr( 10 )
local nHandle   := fcreate( cInstruct, 0 )

fwrite( nHandle, 'open ' + ip + CRLF )
fwrite( nHandle, login + CRLF )
fwrite( nHandle, password + CRLF )
fwrite( nHandle, 'cd ' + path + CRLF )
fwrite( nHandle, 'ls -l' + CRLF )
fwrite( nHandle, 'quit' + CRLF )
fclose( nHandle )

? path

swpruncmd( 'ftp -s:' + cInstruct + ' > ' + cDirectry )
readdir( cDirectry, dbfname )

return

*+
*+
*+    Procedure readdir()
*+
*+    Called from ( ftpdir.prg   )   1 - procedure ftpdir()
*+
*+
*+
procedure readdir( inputfile, dbfname )

local cThisLine
local aToken

if bopen( inputfile )

   dbcreate( DBFNAME, { { 'FILENAME', 'C', 40, 0 }, ;
                        { 'DATE'    , 'D',  8, 0 }, ;
                        { 'TIME'    , 'C',  5, 0 }, ;
                        { 'SIZE'    , 'N', 10, 0 } } )

   use ( dbfname ) exclusive new alias thetable

   do while !beof()

      cThisLine := breadline()

      aToken := tokenize( cThisLine, ' ' )

      if len( aToken ) > 7 .and. aToken[ 2 ] == '1'

         thetable->( dbappend() )

         thetable->filename := ;
                 upper( strtran( aToken[ 9 ], chr( 13 ) ) )
         thetable->date := ;
                 convert( aToken[ 6 ], aToken[ 7 ], aToken[ 8 ] )
         thetable->time := ;
                 if( ':' $ aToken[ 8 ], aToken[ 8 ], '' )
         thetable->size := val( aToken[ 5 ] )

      endif

   enddo

   thetable->( dbclosearea() )

   bclose()

endif

return

*+
*+
*+    Function convert()
*+
*+    Called from ( ftpdir.prg   )   1 - procedure readdir()
*+
*+
*+
function convert( cMonth, cDay, cYear )

local aMonths := { 'JAN', 'FEB', 'MAR', 'APR', 'MAY', 'JUN', ;
                   'JUL', 'AUG', 'SEP', 'OCT', 'NOV', 'DEC' }

local nPointer := ascan( aMonths, upper( cMonth ) )

cDay := strzero( val( cDay ), 2 )

cMonth := strzero( nPointer, 2 )

if len( cDay ) == 1
   cDay := '0' + cDay
endif

if ':' $ cYear
   cYear := str( year( date() ), 4 )
else
   cYear := right( cYear, 4 )
endif

if stod( cYear + cMonth + cDay ) > date()
   cYear := str( val( cYear ) - 1, 4 )
endif

return stod( cYear + cMonth + cDay )

*+
*+
*+    Function token()
*+
*+    Called from ( ftpdir.prg   )   1 - function tokenize()
*+
*+
*+
function token( cString, cDelimiter, nPointer )

local x
local nStrLen      := len( cString )
local nHowMany     := 0
local nLastPointer := 0
local cPart
local lFoundWord   := .f.

default cDelimiter to ' ,.;:!?/\<>()^#&%+-*' + chr( 0 ) + chr( 9 ) + ;
        chr( 10 ) + chr( 13 ) + ;
        chr( 26 ) + chr( 138 ) + ;
        chr( 141 )

for x := 1 to nStrLen
   if substr( cString, x, 1 ) $ cDelimiter
      if lFoundWord
         nHowMany ++
      endif
      if nHowMany == nPointer
         exit
      endif
      do while x < nStrLen .and. substr( cString, x + 1, 1 ) $ cDelimiter
         x ++
      enddo
      nLastPointer := x
   else
      lFoundWord := .t.
   endif
next

if x == nStrLen + 1                     // went all the way without nHowmany == nPointer
   // take the last word
   cPart := substr( cString, nLastPointer + 1 )
else
   cPart := substr( cString, nLastPointer + 1, ( x - nLastPointer ) - 1 )
endif

return cPart

*+
*+
*+    Function numtoken()
*+
*+    Called from ( ftpdir.prg   )   1 - function tokenize()
*+
*+
*+
function numtoken( cString, cDelimiter )

local x
local nStrLen    := len( cString )
local nHowMany   := 0
local lFoundWord := .f.
local retval     := 0

default cDelimiter to ' ,.;:!?/\<>()^#&%+-*' + chr( 0 ) + chr( 9 ) + ;
        chr( 10 ) + chr( 13 ) + ;
        chr( 26 ) + chr( 138 ) + ;
        chr( 141 )

for x := 1 to nStrLen
   if substr( cString, x, 1 ) $ cDelimiter
      if lFoundWord
         nHowMany ++
      endif
      do while x < nStrLen .and. substr( cString, x + 1, 1 ) $ cDelimiter
         x ++
      enddo
   else
      lFoundWord := .t.
   endif
next

if nStrLen > 0
   retval := ;
           iif( right( cString, 1 ) $ cDelimiter, nHowmany, nHowMany + 1 )
endif

return retval

*+
*+
*+    Function tokenize()
*+
*+    Called from ( ftpdir.prg   )   1 - procedure readdir()
*+
*+
*+
function tokenize( cString, cDelimiter )

local nTokens := numtoken( cString, cDelimiter )
local aString := array( nTokens )
local x

for x := 1 to nTokens
   aString[ x ] := token( cString, cDelimiter, x )
next

return aString

*+
*+
*+    Function stod()
*+
*+    Called from ( ftpdir.prg   )   2 - function convert()
*+
*+
*+
function stod( cDate )

local dRetVal     := ctod( '' )
local aDateFormat := gdateformt()

__SetCentury( .T. )
set( _SET_DATEFORMAT, "yyyy.mm.dd" )    // ANSI

cDate := alltrim( cDate )

if len( cDate ) == 8

   dRetVal := ctod( left( cDate, 4 ) + '.' + ;
                    substr( cDate, 5, 2 ) + '.' + ;
                    substr( cDate, 7, 2 ) )

endif

sdateformt( aDateFormat )

return dRetVal

*+
*+
*+    Function gdateformt()
*+
*+    Called from ( ftpdir.prg   )   1 - function stod()
*+
*+
*+
function gdateformt()

return { __SetCentury(), set( _SET_DATEFORMAT ), set( _SET_EPOCH ) }

*+
*+
*+    Procedure sdateformt()
*+
*+    Called from ( ftpdir.prg   )   1 - function stod()
*+
*+
*+
procedure sdateformt( aDateSet )

__SetCentury( aDateSet[ 1 ] )
set( _SET_DATEFORMAT, aDateSet[ 2 ] )
set( _SET_EPOCH, aDateSet[ 3 ] )

return

*+ EOF: FTPDIR.PRG
